/*
 * main.cc
 *
 *  Created on: Feb 6, 2021
 *      Author: sgupta
 */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif
#include<math.h>
#include<iostream>
#include<fstream>
#include<vector>
#include<map>
#include<string>
#include<stdlib.h>
#include<time.h>
#include<exception>
#include<chrono>

//#define S_FV

#include"duneincludes.hh"
#include"parameters.hh"
#include"lop.hh"
#include"top.hh"
#include"driver.hh"

int main(int argc, char** argv)
{
	  try{
	    // initialize MPI
	    Dune::MPIHelper& helper = Dune::MPIHelper::instance(argc, argv);
	    if (argc!=2) return 1;

	    // INI-FILE FOR USER-DEFINED INPUTS
	    char INI_FILE[100];
	    sscanf(argv[1],"%99s", INI_FILE);
	    std::string input_file = INI_FILE;
        if(helper.rank()==0){
	    std::cout<< "input file: " << input_file << std::endl ;
        }

        // PARAMETER TREE
	    Dune::ParameterTree ptree;
	    Dune::ParameterTreeParser ptreeparser;
	    ptreeparser.readINITree(input_file,ptree);
	    ptreeparser.readOptions(argc,argv,ptree);

	    //YASP MESH
	    const int dim =2;
	    Dune::FieldVector<double,dim> L;
        L[0] = ptree.get("domain.length.x",(double)1.);
        L[1] = ptree.get("domain.length.y",(double)0.1);
        std::array<int,dim> N;
        N[0] = ptree.get("domain.num_cells.x",(int)100);
        N[1] = ptree.get("domain.num_cells.y",(int)10);
		std::bitset<dim> periodic(false);
		int overlap=1;
        typedef Dune::YaspGrid<dim> Grid;
        std::shared_ptr<Grid> grid = std::shared_ptr<Grid>(new Grid(L,N,periodic,overlap,helper.getCommunicator()));

        //GRIDVIEW
        typedef Grid::LeafGridView GV;
        GV gv=grid->leafGridView();
        grid->loadBalance();

        //PROBLEM PARAMETERS
    	using Parameters = DefaultParameters<GV,Dune::ParameterTree>;
    	Parameters parameter(gv,ptree);

    	//PATH+FILE NAMES FOR VTK OUTPUT
    	auto pathname = ptree.get("output.path_name",(std::string)"__PATH__");
    	auto filename = ptree.get("output.file_name",(std::string)"test_0");

        //PROLEM DRIVER
		driver(gv,ptree,parameter,pathname,filename,helper);

	  }
	  catch (Dune::Exception &e){
	    std::cerr << "Dune reported error: " << e << std::endl;
	  }
	  catch (...){
	    std::cerr << "Unknown exception thrown!" << std::endl;
	  }
}
